﻿using Newtonsoft.Json;
using Syncfusion.DocIO;
using Syncfusion.DocIO.DLS;
using Syncfusion.DocToPDFConverter;
using Syncfusion.EJ.PdfViewer;
using Syncfusion.OfficeChartToImageConverter;
using Syncfusion.Pdf;
using Syncfusion.Pdf.Parsing;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web;
using System.Web.Http;

namespace ASPPDFDemo
{
    public class PdfViewerController : ApiController
    {
        public object Load(Dictionary<string, string> jsonResult)
        {
            PdfViewerHelper helper = new PdfViewerHelper();
            //load the multiple document from client side             
            if (jsonResult.ContainsKey("newFileName"))
            {
                var name = jsonResult["newFileName"];
                var pdfName = name.ToString();
                helper.Load(Helper.GetFilePath("" + name));
            }
            else
            {
                if (jsonResult.ContainsKey("isInitialLoading"))
                {
                    if (jsonResult.ContainsKey("file"))
                    {
                        var name = jsonResult["file"];
                        helper.Load(name);
                    }
                    else
                    {
                        //Load an existing Word document

                        WordDocument wordDocument = new WordDocument(Helper.GetFilePath("HTTP Succinctly.docx"), FormatType.Docx);

                        //Initialize chart to image converter for converting charts during Word to pdf conversion

                        wordDocument.ChartToImageConverter = new ChartToImageConverter();

                        //Create an instance of DocToPDFConverter

                        DocToPDFConverter converter = new DocToPDFConverter();

                        //Convert Word document into PDF document

                        PdfDocument pdfDocument = converter.ConvertToPDF(wordDocument);

                        MemoryStream ms = new MemoryStream();

                        pdfDocument.Save(ms);                       

                        helper.Load(ms);
                                        
                    }
                }
            }
            string output = JsonConvert.SerializeObject(helper.ProcessPdf(jsonResult));
            return output;
        }
        public object FileUpload(Dictionary<string, string> jsonResult)
        {
            PdfViewerHelper helper = new PdfViewerHelper();

            if (jsonResult.ContainsKey("uploadedFile"))
            {
                var fileurl = jsonResult["uploadedFile"];
                byte[] byteArray = Convert.FromBase64String(fileurl);
                MemoryStream stream = new MemoryStream(byteArray);
                helper.Load(stream);
            }

            string output = JsonConvert.SerializeObject(helper.ProcessPdf(jsonResult));
            return output;
        }
        public object Download(Dictionary<string, string> jsonResult)
        {
            PdfViewerHelper helper = new PdfViewerHelper();            
            return (helper.GetDocumentData(jsonResult));
        }
    }

    public class Helper
    {
        public static string GetFilePath(string path)
        {
            string _dataPath = GetCommonFolder(new DirectoryInfo(HttpContext.Current.Request.PhysicalApplicationPath));
            _dataPath += @"\" + path;
            return _dataPath;
        }

        static string GetCommonFolder(DirectoryInfo dtInfo)
        {
            var _folderNames = dtInfo.GetDirectories("Data");
            if (_folderNames.Length > 0)
            {
                return _folderNames[0].FullName;
            }

            return dtInfo.Parent != null ? GetCommonFolder(dtInfo.Parent) : string.Empty;
        }
    }
}